@@ -3,6 +3,8 @@
 import os.path
 import logging
 import tempfile
+from typing import List, Union
+
 from deprecated import deprecated
 from shutil import rmtree
 from datetime import datetime
@@ -13,6 +15,10 @@ from selenium.webdriver.firefox.options import Options as FfOptions
 from selenium.webdriver.edge.options import Options as EdgOptions
 from selenium.webdriver.opera.options import Options as OpeOptions
 from selenium.webdriver.remote.webelement import WebElement
+# Driver's Service
+from selenium.webdriver.chrome.service import Service as ChrService
+from selenium.webdriver.firefox.service import Service as FfService
+from selenium.webdriver.edge.service import Service as EdgService
 # Driver Manager
 from webdriver_manager.chrome import ChromeDriverManager
 from webdriver_manager.firefox import GeckoDriverManager
@@ -23,11 +29,12 @@ from webdriver_manager.utils import ChromeType
 from .navigators import go_to_url, enter_frame, go_to_window
 from .finders import find_element, find_elements, find_from_elements, \
     find_sub_element_from_element
-from .actions import fill_element, fill_elements, select_in_dropdown, set_checkbox, \
+from .actions import fill_element, fill_elements, mouse_click, select_in_dropdown, set_checkbox, \
     click_element, select_in_angular_dropdown, hover_element, select_in_elements
 from .alerts import alert_message, intercept_alert
 from .information import is_alert_present, is_field_exist, is_field_contains_text, \
-    element_text, is_field_displayed, is_field_enabled, how_many_windows, where_am_i, \
+    element_text, is_field_displayed, is_field_enabled, how_many_windows, wait_for_another_window, \
+    where_am_i, \
     is_checkbox_checked, retrieve_tabular
 from .drivers_tools import fullpage_screenshot, move_to
 
@@ -53,28 +60,39 @@ class BrowserServer:
         "opera": OpeOptions,
         "edge": EdgOptions
     }
-    
+
+    __SERVICE_SWITCHER = {
+        "chrome": ChrService,
+        "headless-chrome": ChrService,
+        "chromium": ChrService,
+        "headless-chromium": ChrService,
+        "firefox": FfService,
+        "opera": ChrService,
+        "edge": EdgService
+    }
+
     __DRIVER_MANAGER = {
-            "chrome": ChromeDriverManager,
-            "headless-chrome": ChromeDriverManager,
-            "chromium": ChromeDriverManager,
-            "headless-chromium": ChromeDriverManager,
-            "firefox": GeckoDriverManager,
-            "edge": EdgeChromiumDriverManager,
-            "opera": OperaDriverManager
-        }
-    
+        "chrome": ChromeDriverManager,
+        "headless-chrome": ChromeDriverManager,
+        "chromium": ChromeDriverManager,
+        "headless-chromium": ChromeDriverManager,
+        "firefox": GeckoDriverManager,
+        "edge": EdgeChromiumDriverManager,
+        "opera": OperaDriverManager
+    }
+
     __WEB_DRIVERS = {
-            "chrome": webdriver.Chrome,
-            "headless-chrome": webdriver.Chrome,
-            "chromium": webdriver.Chrome,
-            "headless-chromium": webdriver.Chrome,
-            "firefox": webdriver.Firefox,
-            "edge": webdriver.Edge,
-            "safari": webdriver.Safari,
-            "opera": webdriver.Opera
-        }
-    
+        "chrome": webdriver.Chrome,
+        "headless-chrome": webdriver.Chrome,
+        "chromium": webdriver.Chrome,
+        "headless-chromium": webdriver.Chrome,
+        "firefox": webdriver.Firefox,
+        "edge": webdriver.Edge,
+        "safari": webdriver.Safari,
+        "opera": webdriver.Opera
+    }
+
+
     def __init__(self):
         # Definition of private attributes and references
         # All strings allowed for the browser name and version
@@ -166,7 +184,10 @@ class BrowserServer:
         :return: 0 if success
         """
         # todo use the data in order to launch the expected webdriver
-        if self.browser_name == "safari":
+        if self.browser_name is None:
+            raise AttributeError("You must set a browser name. "
+                                 f"Use one of '{self.__authorized_name_version}'")
+        elif self.browser_name == "safari":
             self.__web_driver = BrowserServer.__WEB_DRIVERS[self.browser_name](
                 executable_path=self.driver_path)
         elif "headless-chrom" in self.browser_name:
@@ -181,11 +202,13 @@ class BrowserServer:
                     self.__driver_path = BrowserServer.__DRIVER_MANAGER[self.browser_name](
                         chrome_type=ChromeType.CHROMIUM).install()
                 else:
-                    self.__driver_path = BrowserServer.__DRIVER_MANAGER[self.browser_name]().install()
+                    self.__driver_path = BrowserServer.__DRIVER_MANAGER[
+                        self.browser_name]().install()
+            service = BrowserServer.__SERVICE_SWITCHER[self.browser_name](self.__driver_path)
             self.__web_driver = BrowserServer.__WEB_DRIVERS[self.browser_name](
-                    executable_path=self.__driver_path,
-                    options=option
-                )
+                service=service,
+                options=option
+            )
         elif "chrom" in self.browser_name:
             # Hack https://stackoverflow.com/questions/64927909/
             # failed-to-read-descriptor-from-node-connection-a-device-attached-to-the-system
@@ -194,16 +217,18 @@ class BrowserServer:
                     self.__driver_path = BrowserServer.__DRIVER_MANAGER[self.browser_name](
                         chrome_type=ChromeType.CHROMIUM).install()
                 else:
-                    self.__driver_path = BrowserServer.__DRIVER_MANAGER[self.browser_name]().install()
+                    self.__driver_path = BrowserServer.__DRIVER_MANAGER[
+                        self.browser_name]().install()
             option = BrowserServer.__OPTIONS_SWITCHER[self.browser_name]()
             option.add_experimental_option('excludeSwitches', ['enable-logging'])
             if self.driver_options:
                 for opt in self.driver_options:
                     option.add_argument(opt)
+            service = BrowserServer.__SERVICE_SWITCHER[self.browser_name](self.__driver_path)
             self.__web_driver = BrowserServer.__WEB_DRIVERS[self.browser_name](
-                    executable_path=self.__driver_path,
-                    options=option
-                )
+                service=service,
+                options=option
+            )
         elif self.browser_name is not None:
             if self.__driver_path is None:
                 self.__driver_path = BrowserServer.__DRIVER_MANAGER[self.browser_name]().install()
@@ -212,9 +237,10 @@ class BrowserServer:
             if self.driver_options:
                 for opt in self.driver_options:
                     option.add_argument(opt)
+            service = BrowserServer.__SERVICE_SWITCHER[self.browser_name](self.__driver_path)
             self.__web_driver = BrowserServer.__WEB_DRIVERS[self.browser_name](
-                    executable_path=self.__driver_path,
-                    options=option)
+                service=service,
+                options=option)
 
         else:
             raise AttributeError("You must set a browser name. "
@@ -272,7 +298,7 @@ class BrowserServer:
 
     # Start of convenient usage of the automaton tools
     # Navigation
-    def go_to(self, url=None):
+    def go_to(self, url: str = None):
         """Navigate to the given url"""
         return go_to_url(driver=self.webdriver, url=url)
 
@@ -280,7 +306,7 @@ class BrowserServer:
         """Enter the frame found using the find_element method"""
         return enter_frame(driver=self.webdriver, field=field, web_element=web_element)
 
-    def go_to_window(self, handle=None, title=None):
+    def go_to_window(self, handle: Union[str, int] = None, title: str = None):
         """Switch to a window providing either the window handle or window title"""
         return go_to_window(driver=self.webdriver, handle=handle, title=title)
 
@@ -299,14 +325,17 @@ class BrowserServer:
         """Find elements using the current webdriver or the provided WebElement"""
         return find_elements(driver=self.webdriver, field=field, web_element=web_element)
 
-    def find_from_elements(self, field=None, text=None, web_element: WebElement = None):
+    def find_from_elements(self,
+                           field: dict = None,
+                           text: str = None,
+                           web_element: WebElement = None):
         """Find element from a list of elements based on the text (exact match)"""
         return find_from_elements(driver=self.webdriver,
                                   field=field,
                                   text=text,
                                   web_element=web_element)
 
-    @deprecated(version="1.0.5", reason="You should user find_element with a web_element")
+    @deprecated(version="1.0.5", reason="You should use find_element with a web_element")
     def find_sub_element_from_element(self, field=None):
         """
         Find the webelement contained in another webelement.
@@ -320,14 +349,14 @@ class BrowserServer:
         return find_sub_element_from_element(element, field=field)
 
     # Actions
-    def fill_element(self, field=None, web_element=None, value=None):
+    def fill_element(self, field: dict = None, web_element: WebElement = None, value: str = None):
         """Fill the element with the value. Use the find_element method to find it"""
         return fill_element(driver=self.webdriver,
                             field=field,
                             web_element=web_element,
                             value=value)
 
-    def fill_elements(self, fields=None, web_element=None, data=None):
+    def fill_elements(self, fields: dict = None, web_element: WebElement = None, data: dict = None):
         """Fill data in the respective field. data and fields keys must match.
         data keys must be in field keys"""
         return fill_elements(driver=self.webdriver,
@@ -336,14 +365,20 @@ class BrowserServer:
                              data=data)
 
     # TODO add unit test
-    def click_element(self, field=None, web_element=None):
+    def click_element(self, field: dict = None, web_element: WebElement = None):
         """Perform a click element on the element. Use the find_element method to find it"""
         return click_element(driver=self.webdriver,
                              field=field,
                              web_element=web_element)
 
-    def select_in_dropdown(self, field=None, visible_text=None, value=None):
+    def mouse_click(self, field: dict = None, web_element: WebElement = None):
+        """Perform a mouse click on the field. Use the find_element method to find if"""
+        return mouse_click(driver=self.webdriver,
+                           field=field,
+                           web_element=web_element)
 
+    def select_in_dropdown(self, field: dict = None, visible_text: str = None, value: str = None):
+        """Select in field dropdown either the option by its visible text or its value"""
         return select_in_dropdown(driver=self.webdriver,
                                   field=field,
                                   visible_text=visible_text,
@@ -358,82 +393,151 @@ class BrowserServer:
 
     # TODO add unit test
     def select_in_angular_dropdown(self, root_field=None, visible_text=None):
+        """Select a field within a mat_option list
+
+    If root_field is defined then click it and search for visible_text in mat-option elements.
+    Otherwise only search for visible_text in mat-options elements"""
         return select_in_angular_dropdown(driver=self.webdriver,
                                           root_field=root_field,
                                           visible_text=visible_text)
 
-    def set_checkbox(self, field=None, is_checked=None):
+    def set_checkbox(self, field: dict = None,
+                     is_checked: bool = None,
+                     web_element: WebElement = None):
+        """Set the field checkbox to the is_checked state"""
         return set_checkbox(driver=self.webdriver,
                             field=field,
-                            is_checked=is_checked)
+                            is_checked=is_checked,
+                            web_element=web_element)
 
     # TODO add unit test
-    def hover_element(self, field=None):
+    def hover_element(self, field: dict = None):
         return hover_element(driver=self.webdriver, field=field)
 
     # TODO add unit test
-    def select_in_elements(self, field=None, displayed_text=None):
+    def select_in_elements(self,
+                           field: dict = None,
+                           displayed_text: str = None,
+                           web_element: WebElement = None) -> int:
+        """Do a click on the element which text match"""
         return select_in_elements(driver=self.webdriver,
                                   field=field,
-                                  displayed_text=displayed_text)
+                                  displayed_text=displayed_text,
+                                  web_element=web_element)
 
-    def execute_script(self, script: str, web_element: WebElement):
-        self.webdriver.execute_script(script, web_element)
+    def execute_script(self, script: str, *args):
+        """Delegate to the current webdriver the execute_script.
+         See :func:selenium.webdriver.remote.webdriver.execute_script
+        """
+        self.webdriver.execute_script(script, *args)
 
     # Alerts
     def alert_message(self):
+        """Return the message displayed in the alert."""
         return alert_message(driver=self.webdriver)
 
-    def intercept_alert(self, messages=None, accept=True, value=None):
+    def intercept_alert(self, messages: List[str] = None, accept: bool = True, value: str = None):
+        """intercept_alert goal is to provide an easy interface to handle alerts.
+
+        Without arguments, accept the alert whatever the message is.
+
+        Giving a list of messages, ensure that the alert message is one of the list.
+
+        Giving value, insert it in the prompt field.
+        """
         return intercept_alert(driver=self.webdriver, messages=messages,
                                accept=accept, value=value)
 
     # Information
 
-    def is_alert_present(self, until=5):
+    def is_alert_present(self, until: int = 5):
+        """Tells if whatever an alert is present or not."""
         return is_alert_present(driver=self.webdriver, until=until)
 
-    def is_field_exist(self, field=None, web_element=None, until=5, avoid_move_to=True):
+    def is_field_exist(self,
+                       field: dict = None,
+                       web_element: WebElement = None,
+                       until: int = 5,
+                       avoid_move_to: bool = True):
+        """Check the field existence in the DOM. Unless specified doesn't bring into view the
+        element"""
         return is_field_exist(driver=self.webdriver,
                               field=field,
                               web_element=web_element,
                               until=until,
                               avoid_move_to=avoid_move_to)
 
-    def is_field_contains_text(self, field=None, web_element=None, text=None):
+    def is_field_contains_text(self,
+                               field: dict = None,
+                               web_element: WebElement = None,
+                               text: str = None) -> bool:
+        """Test if the field contains the text."""
         return is_field_contains_text(driver=self.webdriver,
                                       field=field,
                                       web_element=web_element,
                                       text=text)
 
-    def element_text(self, field=None, web_element=None):
+    def element_text(self,
+                     field: dict = None,
+                     web_element: WebElement = None,
+                     avoid_move_to: bool = False) -> Union[str, None]:
+        """Return the element text as string or None if field is not found"""
         return element_text(driver=self.webdriver,
                             field=field,
-                            web_element=web_element)
+                            web_element=web_element,
+                            avoid_move_to=avoid_move_to)
 
-    def is_field_displayed(self, field=None, web_element=None, avoid_move_to=True):
+    def is_field_displayed(self,
+                           field: dict = None,
+                           web_element: WebElement = None,
+                           avoid_move_to: bool = True,
+                           wait_until: int = 5) -> bool:
+        """Check if field is displayed"""
         return is_field_displayed(driver=self.webdriver,
                                   field=field,
                                   web_element=web_element,
-                                  avoid_move_to=avoid_move_to)
+                                  avoid_move_to=avoid_move_to,
+                                  wait_until=wait_until)
 
-    def is_field_enabled(self, field=None, web_element=None, attribute=None):
-        return is_field_enabled(driver=self.webdriver, field=field, web_element=web_element,
-                                attribute=attribute)
+    def is_field_enabled(self,
+                         field: dict = None,
+                         web_element: WebElement = None,
+                         attribute: str = None,
+                         avoid_move_to: bool = False,
+                         wait_until: int = 5) -> bool:
+        """Check if the field is enabled"""
+        return is_field_enabled(driver=self.webdriver,
+                                field=field,
+                                web_element=web_element,
+                                attribute=attribute,
+                                avoid_move_to=avoid_move_to,
+                                wait_until=wait_until)
 
-    def how_many_windows(self):
+    def how_many_windows(self) -> int:
+        """Current number of handles"""
         return how_many_windows(driver=self.webdriver)
 
-    def retrieve_tabular(self, field=None, web_element=None, row_and_col=("tr", "td", "th")):
+    def retrieve_tabular(self,
+                         field: dict = None,
+                         web_element: WebElement = None,
+                         row_and_col: tuple = ("tr", "td", "th")) -> List[list]:
+        """Retrieve a tabular as a list of list of cells' content (str)"""
         return retrieve_tabular(driver=self.webdriver, field=field, web_element=web_element,
                                 row_and_col=row_and_col)
 
     # TODO add unit test
-    def where_am_i(self):
+    def where_am_i(self) -> str:
+        """Current URL"""
         return where_am_i(driver=self.webdriver)
 
     # TODO add unit test
-    def is_checkbox_checked(self, field=None, web_element=None, is_angular=False):
+    def is_checkbox_checked(self,
+                            field: dict = None,
+                            web_element: WebElement = None,
+                            is_angular: bool = False):
         return is_checkbox_checked(driver=self.webdriver, field=field, web_element=web_element,
                                    is_angular=is_angular)
 
+    def wait_for_another_window(self, wait_until: int = 1) -> bool:
+        """Wait for another window to pop"""
+        return wait_for_another_window(driver=self.webdriver, until=wait_until)
